//
//  CKMWindowController.m
//  commandKeyMode
//
//  Created by narita on 2016/07/21.
//  Copyright © 2016年 narita. All rights reserved.
//

#import "CKMWindowController.h"

@interface CKMWindowController ()

@property (strong, readwrite) id eventMonitor;

@end

@implementation CKMWindowController

- (void)windowDidLoad {
    [super windowDidLoad];
    
    
}

// クラス名と同じnibファイルを読み込む

- (NSString *)windowNibName
{
    NSString* theSuperWindowNibName = [super windowNibName];
    
    if(theSuperWindowNibName != nil && ![theSuperWindowNibName isEqualToString:@""])
    {
        return theSuperWindowNibName;
    }
    else
    {
        NSString* theClassName = NSStringFromClass([self class]);
        
        return theClassName;
    }
}

- (void)windowDidBecomeKey:(NSNotification *)notification
{
    NSLog(@"windowDidBecomeKey:");

    [self enterCommandKeyMode];
}

- (void)windowDidResignKey:(NSNotification *)notification
{
    NSLog(@"windowDidResignKey:");
    
    [self exitCommandKeyMode];
}


- (void) enterCommandKeyMode
{
    [self relayoutCommandKeyPanel];
    
    // コマンドモードの準備
    __weak CKMWindowController* weakSelf = self;
    
    self.eventMonitor = [NSEvent addLocalMonitorForEventsMatchingMask:( NSKeyUpMask | NSKeyDownMask | NSFlagsChangedMask )
                                                              handler:^(NSEvent* inIncomingEvent)
                         {
                             NSEvent*       theResult    = inIncomingEvent;
                             NSEventType    theEventType = [inIncomingEvent type];
                             
                             if( inIncomingEvent.window == weakSelf.window )
                             {
                                 NSUInteger     theModifierFlags = [inIncomingEvent modifierFlags];

                                 if (theModifierFlags & NSCommandKeyMask)
                                 {
                                     if ( theEventType == NSKeyDown && inIncomingEvent.isARepeat == NO)
                                     {
                                         NSLog(@"** command Key Down");
                                         
                                         theResult = [weakSelf keyDownCommandKeyMode:inIncomingEvent];
                                     }
                                 }
                                 else
                                 {
                                     [weakSelf exitCommandKeyMode];
                                     theResult = nil;
                                 }
                             }
                             
                             return theResult;
                         }];

}


- (NSEvent*) keyDownCommandKeyMode:(NSEvent*) inIncomingEvent
{
    NSLog(@"inIncomingEvent => %@", inIncomingEvent);

    NSResponder* theResponder = [[NSApp mainWindow] firstResponder];

    if( [theResponder class] == [NSTextView class] )
    {
        NSString* theKey   = [inIncomingEvent characters];
       
        if(
           [theKey isEqualToString:@"l"]
           ||
           [theKey isEqualToString:@"r"]
           ||
           [theKey isEqualToString:@"c"]
           ||
           [theKey isEqualToString:@"j"]
           ||
           [theKey isEqualToString:@"t"]
           ||
           [theKey isEqualToString:@"b"]
           ||
           [theKey isEqualToString:@"u"]
           ||
           [theKey isEqualToString:@"i"]
           ||
           [theKey isEqualToString:@"+"]
           ||
           [theKey isEqualToString:@"="]
           ||
           [theKey isEqualToString:@"-"]
           )
        {
            
        }
        else
        {
            inIncomingEvent = nil;
        }
    }

    return inIncomingEvent;
}

- (void) exitCommandKeyMode
{
    [self close];
    
    // コマンドモードの終了
    if (self.eventMonitor)
    {
        [NSEvent removeMonitor:self.eventMonitor];
        self.eventMonitor = nil;
    }
}

- (void) relayoutCommandKeyPanel
{
    
    // 位置の調整
    NSRect theMainWindowFrame = [[NSApp mainWindow] frame];
    NSRect theSelfWindowFrame = self.window.frame;
    NSScreen* theScreen = [[NSApp mainWindow] screen];
    float theDeltaX = (theMainWindowFrame.size.width - theSelfWindowFrame.size.width) / 2;
    float theDeltaY = (theMainWindowFrame.size.height - theSelfWindowFrame.size.height) / (1.5);
    
    NSRect theNewFrame;
    theNewFrame.origin = theMainWindowFrame.origin;
    theNewFrame.size   = theSelfWindowFrame.size;
    
    theNewFrame.origin.x += theDeltaX;
    theNewFrame.origin.y += theDeltaY;
    
    theNewFrame = [[NSApp mainWindow] constrainFrameRect:theNewFrame toScreen:theScreen];
    
    [self.window setFrame:theNewFrame display:YES];
}

@end
